"""
    Copyright (C) 2021  CNRS
    This file is part of "Speedlib".
    "Speedlib" is an API built for the use case of autonomous navigation.
    It has  been developed to control quay cranes and trains of multimodal
    waterborne Lab as part of The SPEED project, a project which aims to
    enhance and support the growth of a system of connected port solutions,
    with the use of data science and IoT (Internet of Things) technologies.
    The library allows controlling the motion of the IoT devices at H0 scale
    in automatic mode, in three directions and exchanging with the information
    system for overall management
"""
# -*-coding: <Utf-8> -*-
from speedlib.dcc import dcc_object
from speedlib.dcc.dcc_object import DCCObject
import time

class Switch():
    """ This class is used to control Servo motors """
    def __init__(self, name, adress, biais_id):
        """
        Parameters
        ----------
        name : string
            DESCRIPTION : It is the name of the train
        adress : int
            DESCRIPTION This is the address on which it was programmed
        biais_id : int
            DESCRIPTION Allows you to choose which servomotor will be used
        Returns
        -------
        None.

        """
        self.name = name
        self.adress = adress
        self.biais_id = biais_id

        if not isinstance(self.biais_id, int):
            raise TypeError("biais_id must be an int but got "+str(self.biais_id))
        if self.biais_id not in [1, 2]:
            raise ValueError("biais_id must be an 1 or 2 but got "+str(self.biais_id))
        if not isinstance(name, str):
            raise TypeError(" name must be a str but got " +str(name))
        if not isinstance(adress, int):
            raise TypeError("adress must be an integer but got  " +str(adress))
        #if adress not in range(101, 126):
            #raise RuntimeError("""The address must be between 101 and 125 but got """+str(adress))

        self.dccobject = DCCObject(name, adress)

    def _get_biais(self):
        """
        Returns
        -------
        TYPE
            DESCRIPTION : Returns the current state of the switch

        """
        if self.biais_id == 1:
            return self.dccobject.f1, self.dccobject.f_light
        elif self.biais_id == 2:
            return self.dccobject.f2
    
    def set_biais_id(self, id):
        """
        Parameters
        ----------
        var : int
            DESCRIPTION : Allows to change the id of the swicth 
                            that we would like to control

        Returns
        -------
        None.

        """
        self.biais_id = id

    def _set_biais(self, state):
        """
        Parameters
        ----------
        var : Boolean
            DESCRIPTION : change the state of the switch

        Returns
        -------
        None.

        """
        if not isinstance(state, bool):
            raise TypeError(" var must me a boolean but got "+str(state))

        if self.biais_id == 1:
            self.dccobject.f1 = state
            self.dccobject.f_light = state

        elif self.biais_id == 2:
            self.dccobject.f2 = state
            self.dccobject.reverse()


    biais = property(_get_biais, _set_biais)



if __name__ == "__main__":
    S = Switch("DCC3", 3, 2)
    dcc_object.start()
    S.biais = True
    time.sleep(1)
    S.set_biais_id(1)
    time.sleep(1)
    S.biais = False
    dcc_object.stop()
    



"""
    Copyright (C) 2021  CNRS
    This file is part of "Speedlib".
    "Speedlib" is an API built for the use case of autonomous navigation.
    It has  been developed to control quay cranes and trains of multimodal
    waterborne Lab as part of The SPEED project, a project which aims to
    enhance and support the growth of a system of connected port solutions,
    with the use of data science and IoT (Internet of Things) technologies.
    The library allows controlling the motion of the IoT devices at H0 scale
    in automatic mode, in three directions and exchanging with the information
    system for overall management
"""
# -*-coding: <Utf-8> -*-
# -*-coding: <Utf-8> -*-
from dccpi import *

e = DCCRPiEncoder()
controller = DCCController(e) #Create the DCC controller with the RPi encoder

def start():
    """ Starts the controller and Removes a brake signal """
    controller.start()
def stop():
    """ This function stops the controller and enables a brake signal on tracks"""
    controller.stop()



class DCCObject(object):
    """ Speedlib librairie to control a locomive device"""
    def __init__(self, name, adress):
        """ this function takes a name and an address (an integer # 0) as
        parameters to create a train and  register it  on the controller
        """
        self.name = name
        self.adress = adress
        self._l = DCCLocomotive(name, adress)  # Create locos, args: Name, DCC Address
        controller.register(self._l)  # Register locos on the controller
        self.speed = 0
        self.fl = False
        self.f1 = False
        self.f2 = False
        self.f3 = False
        self.f4 = False

    def reverse(self):
        """Change the direction"""
        self._l.reverse()

    def faster(self):
        """ Increase 1 speed step"""
        self._l.faster()

    def slower(self):
        """Reduce the speed"""
        self._l.slower()

    def _get_speed(self):
        """ Returns the current speed of the train """
        return self._l.speed

    def _set_speed(self, new_speed):
        """ change the speed"""
        if not isinstance(new_speed, int):
            raise TypeError("vew_speed must be an integer but get "+ str(new_speed))
        self._l.speed = new_speed
    speed = property(_get_speed, _set_speed)

    def __repr__(self):
        return """ DCC locomotive : Name = {},
        address = {},
        speed = {}, 
        fl = {}, 
        f1 = {}, 
        f2 = {},
        f3 = {}, 
        f4 = {} """.format(self.name, self.adress,
                           self._l.speed, self._l.fl,
                           self._l.f1, self._l.f2,
                           self._l.f3, self._l.f4)


    def _get_f_light(self):
        """Returns the current state of fl """
        return self._l.fl

    def _set_f_light(self, var):
        """ change the state of fl """
        if not isinstance(var, bool):
            raise TypeError("var must be a bool but got "+str(var))
        self._l.fl = var
    f_light = property(_get_f_light, _set_f_light)

    def _get_f1(self):
        """Returns the current state of f1 """
        return self._l.f1

    def _set_f1(self, var):
        """ change the state of f1 """
        if not isinstance(var, bool):
            raise TypeError("var must be a bool but got "+str(var))
        self._l.f1 = var
    f1 = property(_get_f1, _set_f1)

    def _get_f2(self):
        """Returns the current state of f2 """
        return self._l.f2

    def _set_f2(self, var):
        """ change the state of f2 """
        if not isinstance(var, bool):
            raise TypeError("var must be a bool but got "+str(var))
        self._l.f2 = var
    f2 = property(_get_f2, _set_f2)

    def _get_f3(self):
        """Returns the current state of f3 """
        return self._l.f3

    def _set_f3(self, var):
        """ change the state of f3 """
        if not isinstance(var, bool):
            raise TypeError("var must be a bool but got "+str(var))
        self._l.f3 = var
    f3 = property(_get_f3, _set_f3)

    def _get_f4(self):
        """Returns the current state of f4 """
        return self._l.f4

    def _set_f4(self, var):
        """ change the state of fl """
        if not isinstance(var, bool):
            raise TypeError("var must be a bool but got "+str(var))
        self._l.f4 = var
    f4 = property(_get_f4, _set_f4)



"""
    Copyright (C) 2021  CNRS
    This file is part of "Speedlib".
    "Speedlib" is an API built for the use case of autonomous navigation.
    It has  been developed to control quay cranes and trains of multimodal
    waterborne Lab as part of The SPEED project, a project which aims to
    enhance and support the growth of a system of connected port solutions,
    with the use of data science and IoT (Internet of Things) technologies.
    The library allows controlling the motion of the IoT devices at H0 scale
    in automatic mode, in three directions and exchanging with the information
    system for overall management
"""
# -*-coding: <Utf-8> -*-
import queue
import threading
import re
import time
import requests
from pint import UnitRegistry

TIME_OUT = 3
ureg = UnitRegistry()
MOTOR_SPREADER = 2
MOTOR_CRAB = 1
MOTOR_CHASSIS = 3
MOTOR_DIRECTION_BACKWARD = -1
MOTOR_DIRECTION_FORWARD  = 1




class Crane():
    """ This class is used to control cranes"""
    def __init__(self):
        """
        Creates the crane object. each crane has 3 motors, each with a Thread and a queue

        Returns
        -------
        None.
        """
        self.slave_ip = None
        self.master_ip = None
        self.dict_q = {}
        for i in range(1,4):
            self.dict_q[i] = queue.Queue()
            threading.Thread(target = self.run_start_for, daemon = True, args=(i,)).start()


    def run_start_for(self, id_moteur):
        """
        Parameters
        ----------
        id_moteur : int
            DESCRIPTION: It is the motor number

        Raises
        ------
        RuntimeError
            DESCRIPTION Return a RuntimeError when the number
            of motor is different from 1, 2 or 3

        Returns
        -------
        None.
        """
        if id_moteur not in self.dict_q.keys() :
            raise RuntimeError("motor_id Error")
        while True:
            arguments = self.dict_q[id_moteur].get()
            init_time = time.time()*ureg.second
            while time.time()*ureg.second - init_time < arguments[0]:
                self.start(arguments[1], arguments[2])
                if not self.dict_q[id_moteur].empty():
                     break
            self.stop(arguments[1])
            self.dict_q[id_moteur].task_done()



    def get_motor_info_from_number(self, snr):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : It is the motornumber

        Raises
        ------
        TypeError
            DESCRIPTION : This exception is raised when the snr parameter is not an integer
        RuntimeError
            DESCRIPTION: This exception is raised when the motor number passed in parameter
                        is different from 1, 2 and 3

        Returns
        -------
        TYPE
            DESCRIPTION.
        int
            DESCRIPTION.
        """
        if not isinstance(snr, int):
            raise TypeError(" The value of n must be an integer but got " +str(snr))



        if MOTOR_SPREADER == snr:
            return (self.slave_ip, 2)
        if MOTOR_CRAB == snr:
            return (self.slave_ip, 1)
        if MOTOR_CHASSIS  == snr:
            return (self.master_ip, 1)
        raise RuntimeError("""Invalid motor number, must be MOTOR_SPREADER, MOTOR_CRAB or
                               MOTOR_CHASSIS , got """+str(snr))


    def start (self, snr, turn):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : It is the motor number
        turn : int
            DESCRIPTION : Direction in which we want the motor to go (-1 or 1)
                          MOTOR_DIRECTION_BACKWARD = -1
                          MOTOR_DIRECTION_FORWARD  = 1

        Returns
        -------
        TYPE : string
            DESCRIPTION : Return 'ok' when everything went well

        """

        if turn not in [MOTOR_DIRECTION_FORWARD , MOTOR_DIRECTION_BACKWARD ]:
            raise RuntimeError("""Invalid parameter, turn must be either MOTOR_DIRECTION_FORWARD  or
                                  MOTOR_DIRECTION_BACKWARD . Got """ +str(turn))

        i_p, num_motor = self.get_motor_info_from_number(snr)
        request_answer = requests.get("http://"+i_p+"/startM?sNr="+str(num_motor)+
                                      "&turn="+str(turn), timeout=TIME_OUT)

        if request_answer.status_code !=200:
            raise RuntimeError("Unable to control the motor, "+str(request_answer.status_code))

        if request_answer.text != "ok":
            raise RuntimeError("""Able to controle the motor but got not OK
                                   answer: """+request_answer.text)
        return request_answer.text

    def stop(self, snr):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : It is the motor number

        Returns
        -------
        TYPE : string
            DESCRIPTION : Return 'ok' when everything went well
        """

        i_p, num_motor = self.get_motor_info_from_number(snr)
        request_answer = requests.get("http://"+i_p+"/stopM?sNr="+str(num_motor), timeout=TIME_OUT)
        if request_answer.status_code !=200:
            raise RuntimeError("Unable to control the motor, "+str(request_answer.status_code))

        if request_answer.text != "ok":
            raise RuntimeError("""Able to controle the motor but got not OK answer: """+
                               request_answer.text)
        return request_answer.text


    def step (self, snr, turn):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : It is the motor number
        turn : int
            DESCRIPTION : Direction in which we want the motor to go (-1 or 1)
                          MOTOR_DIRECTION_BACKWARD = -1
                          MOTOR_DIRECTION_FORWARD  = 1

        Returns
        -------
        TYPE : tuple of string
            DESCRIPTION : Returns the result of the execution of the start and stop functions

        """
        return (self.start(snr, turn), self.stop(snr))


    def start_for(self, time_, snr, turn ):
        """
        Parameters
        ----------
        time_ : float or int
            DESCRIPTION : Represents the time during which we would like to run the motor
        snr : int
            DESCRIPTION : It is the motor number
        turn : int
            DESCRIPTION : Direction in which we want the motor to go (-1 or 1)
                          MOTOR_DIRECTION_BACKWARD = -1
                          MOTOR_DIRECTION_FORWARD  = 1

        Returns
        -------
        None.

        Example
        -------
        start_for(5000*ureg.nanosecond,MOTOR_CHASSIS ,MOTOR_DIRECTION_FORWARD )
        """

        if snr not in [MOTOR_CHASSIS , MOTOR_SPREADER, MOTOR_CRAB]:
            raise RuntimeError("""Invalid parameter, sNr must be either MOTOR_CHASSIS  or
                                MOTOR_SPREADER or MOTOR_CRAB . Got """ +str(snr))

        if time_ < 0:
            raise ValueError("t must be greater than 0 but got "+str(time_))

        arguments = [time_, snr, turn]
        self.dict_q[snr].put(arguments)

    def fswitch(self, value):
        """
        Parameters
        ----------
        value : int
            DESCRIPTION.

        Returns
        -------
        TYPE : string
            DESCRIPTION : eturn 'ok' when everything went well

        """
        request_answer=requests.get("http://"+self.slave_ip+"/fswitch?nr=3"+"&v="+str(value)+
                                    "&dim=1", timeout=TIME_OUT)
        return request_answer.text

    def _get_battery(self):
        """
        Returns
        -------
        TYPE int
            DESCRIPTION it is the state of the baterries
        """
        answer_1 = requests.get("http://"+self.master_ip+"/getBat?n=1", timeout=TIME_OUT)
        answer_2 = requests.get("http://"+self.slave_ip+"/getBat?n=2", timeout=TIME_OUT)

        if answer_1.status_code != 200:
            raise RuntimeError("Please check if the battery  1 is correctly powered")

        if answer_2.status_code != 200:
            raise RuntimeError("Please check if the battery 2 is correctly powered")

        return (int(answer_1.text), int(answer_2.text))

    battery = property(_get_battery)

    def change_speed(self, snr, diff):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : it is the motor number
        diff : int
            DESCRIPTION : It is used to vary the speed of the motor

        Returns
        -------
        num : int
            DESCRIPTION : returns the value of the current speed of the motor

        """
        if not isinstance (diff, int):
            raise TypeError("diff must be an int but got "+str(diff))

        i_p, num_motor = self.get_motor_info_from_number(snr)
        request_answer = requests.get("http://"+i_p+"/changePWMTV?sNr="+str(num_motor)+"&diff="+
                                      str(diff), timeout=TIME_OUT)

        if request_answer.status_code != 200:
            raise RuntimeError(""""Unable to change the speed of the motor,"""+
                                str(request_answer.status_code))

        result = re.search("neuer Speed=\s+(\d+)", request_answer.text)
        num = int(result.groups()[0])
        return num

    def get_speed(self , snr):
        """
        Parameters
        ----------
        snr : TYPE
            DESCRIPTION :  is the motor number

        Returns
        -------
        TYPE : int
            DESCRIPTION : Returns the current speed for a motor

        """

        return self.change_speed(snr, 0)

    def set_speed(self , snr, speed):
        """
        Parameters
        ----------
        snr : int
            DESCRIPTION : is the motor number
        speed : int
            DESCRIPTION : set the speed for a motor

        Returns
        -------
        TYPE : int
            DESCRIPTION : returns the value of the current speed of the motor

        """
        current = self.change_speed(snr, 0)
        diff = speed - current
        return self.change_speed(snr, diff)

    def init (self, i_p):
        """
        Parameters
        ----------
        i_p : string
            DESCRIPTION :
        The purpose of this function is to initialize the IP address of
        the master and once the IP address of the master is
        initialized to obtain that of the slave thanks to the getOtherEsp function

        Returns
        -------
        None.

        """

        self.master_ip = i_p
        self.slave_ip = self.get_other_esp(i_p)

    def get_other_esp(self, i_p):
        """
        Parameters
        ----------
        i_p : string
            DESCRIPTION :  This function has the role of launching a request
                            to obtain the IP address of the slave

        Returns
        -------
        TYPE : string
            DESCRIPTION : Return 'ok' when everything went well

        """
        request_answer = requests.get("http://" + i_p + "/getOtherEsp", timeout = TIME_OUT)
        if request_answer.status_code != 200:
            raise RuntimeError ("""I failed to get the IP address of the slave.
            Check if the latter is correctly supplied then try again""")
        return request_answer.text



if __name__ == "__main__":


    i_p1 = "172.17.217.217"
    i_p2 = "172.17.217.217"

    crane_1 = Crane()
    crane_2 = Crane()

    crane_1.init(i_p1)
    crane_2.init(i_p1)


    crane_1.start_for(1*ureg.second, MOTOR_SPREADER, MOTOR_DIRECTION_BACKWARD )
    crane_2.start_for(1*ureg.second, MOTOR_CRAB, MOTOR_DIRECTION_FORWARD )

    crane_1.start_for(2*ureg.second, MOTOR_CRAB, MOTOR_DIRECTION_FORWARD )
    print(crane_1.fswitch(10))
    print(crane_1.battery)
    print(crane_1.change_speed(MOTOR_CRAB, -40))
    crane_2.start_for(0.01*ureg.second, MOTOR_CHASSIS , MOTOR_DIRECTION_FORWARD )

    print(crane_2.change_speed(MOTOR_SPREADER,20))
    print(crane_2.battery)
    crane_2.step(MOTOR_CHASSIS  ,MOTOR_DIRECTION_BACKWARD )
    crane_1.step(MOTOR_CRAB, MOTOR_DIRECTION_BACKWARD )
    print (crane_1.get_speed(MOTOR_CRAB))



"""
    Copyright (C) 2021  CNRS
    This file is part of "Speedlib".
    "Speedlib" is an API built for the use case of autonomous navigation.
    It has  been developed to control quay cranes and trains of multimodal
    waterborne Lab as part of The SPEED project, a project which aims to
    enhance and support the growth of a system of connected port solutions,
    with the use of data science and IoT (Internet of Things) technologies.
    The library allows controlling the motion of the IoT devices at H0 scale
    in automatic mode, in three directions and exchanging with the information
    system for overall management
"""
# -*-coding: <Utf-8> -*-
from speedlib.dcc import dcc_object
from speedlib.dcc.dcc_object import DCCObject


class Train():
    """
        Train class
    """
    def __init__(self, name, adress):
        """ this function takes a name and an address (an integer # 0) as
        parameters to create a train and  register it  on the controller
        """
        if not isinstance(name, str):
            raise TypeError(" name must be a str but got " +str(name))
        if not isinstance(adress, int):
            raise TypeError("adress must be an integer but got  " +str(adress))
        if adress not in range(1, 100):
            raise RuntimeError("""The address must be between 1 and 127 but
              got """+str(adress))
        self.dccobject = DCCObject(name, adress)

    def faster(self):
        """ Increase 1 speed step"""
        self.dccobject.faster()

    def reverse(self):
        """Change the direction"""
        self.dccobject.reverse()

    def slower(self):
        """Reduce the speed"""
        self.dccobject.slower()

    def __repr__(self):
        return self.dccobject.__repr__()

    def _get_speed(self):
        """ Returns the current speed of the train """
        return self.dccobject.speed

    def _set_speed(self, new_speed):
        """ change the speed"""
        self.dccobject.speed = new_speed
    speed = property(_get_speed, _set_speed)

    def _get_f_light(self):
        """Returns the current state of fl """
        return self.dccobject.fl

    def _set_f_light(self, var):
        """ change the state of fl """
        self.dccobject.fl = var
    f_light = property(_get_f_light, _set_f_light)

    def _get_f1(self):
        """Returns the current state of f1 """
        return self.dccobject.f1

    def _set_f1(self, var):
        """ change the state of f1 """
        self.dccobject.f1 = var
    f1 = property(_get_f1, _set_f1)

    def _get_f2(self):
        """Returns the current state of f1 """
        return self.dccobject.f1

    def _set_f2(self, var):
        """ change the state of f1 """
        self.dccobject.f2 = var
    f2 = property(_get_f2, _set_f2)

    def _get_f3(self):
        """Returns the current state of f1 """
        return self.dccobject.f3

    def _set_f3(self, var):
        """ change the state of f1 """
        self.dccobject.f3 = var
    f3 = property(_get_f3, _set_f3)

    def _get_f4(self):
        """Returns the current state of f1 """
        return self.dccobject.f4

    def _set_f4(self, var):
        """ change the state of f1 """
        self.dccobject.f4 = var
    f4 = property(_get_f4, _set_f4)

if __name__ == "__main__":
    Train_1 = Train("DCC3", 3)
    dcc_object.start()
    Train_1.f_light = True
    print (Train_1)
    dcc_object.stop()
